import { cloneDeep, get, set } from "lodash-es";
import { createValidator, resolveName, toSimpleValidationErrors } from "../utils/validator.mjs";
import { resolveInitialData, resolveString } from "../utils/core.mjs";
import { reactive, ref, toRaw, useNuxtApp } from "#imports";
export function useForm(method, url, inputs, config = {}) {
  const { $precognition } = useNuxtApp();
  const originalData = cloneDeep(toRaw(resolveInitialData(inputs)));
  const originalInputs = Object.keys(originalData);
  const valid = ref([]);
  const touched = ref([]);
  let form;
  const validator = createValidator((client) => client[resolveString(method)](
    resolveString(url),
    form.data(),
    config
  ), resolveInitialData(originalData)).on("validatingChanged", () => {
    form.validating = validator.validating();
  }).on("validatedChanged", () => {
    valid.value = validator.valid();
  }).on("touchedChanged", () => {
    touched.value = validator.touched();
  }).on("errorsChanged", () => {
    form.hasErrors = validator.hasErrors();
    form.errors = toSimpleValidationErrors(validator.errors());
    valid.value = validator.valid();
  });
  const resolveSubmitConfig = (config2) => ({
    ...config2,
    precognitive: false,
    onStart: () => {
      form.processing = true;
      (config2.onStart ?? (() => null))();
    },
    onFinish: () => {
      form.processing = false;
      (config2.onFinish ?? (() => null))();
    },
    onValidationError: (response, error) => {
      if (response._data && typeof response._data === "object" && "errors" in response._data)
        validator.setErrors(response._data.errors);
      if (config2.onValidationError)
        return config2.onValidationError(response);
      throw error ?? new Error("Unknown error.");
    }
  });
  form = {
    ...cloneDeep(toRaw(resolveInitialData(inputs))),
    data() {
      const data = cloneDeep(toRaw(form));
      return originalInputs.reduce((carry, name) => ({
        ...carry,
        [name]: data[name]
      }), {});
    },
    setData(data) {
      Object.keys(data).forEach((input) => {
        if (input in originalData)
          form[input] = data[input];
      });
      return form;
    },
    touched(name) {
      return touched.value.includes(name);
    },
    touch(name) {
      validator.touch(name);
      return form;
    },
    validate(name) {
      if (typeof name === "undefined") {
        validator.validate();
      } else {
        const mainKey = typeof name === "object" ? name.target.name.split(".").at(0) : name.split(".").at(0);
        if (originalInputs.includes(resolveName(mainKey ?? "")))
          validator.validate(name, get(form.data(), name));
      }
      return form;
    },
    validating: false,
    valid(name) {
      return valid.value.includes(name);
    },
    invalid(name) {
      return typeof form.errors[name] !== "undefined";
    },
    errors: {},
    hasErrors: false,
    setErrors(errors) {
      validator.setErrors(errors);
      return form;
    },
    forgetError(name) {
      validator.forgetError(name);
      return form;
    },
    reset(...names) {
      const original = cloneDeep(resolveInitialData(inputs));
      if (names.length === 0)
        originalInputs.forEach((name) => form[name] = original[name]);
      else
        names.forEach((name) => set(form, name, get(original, name)));
      validator.reset(...names);
      return form;
    },
    setValidationTimeout(duration) {
      validator.setTimeout(duration);
      return form;
    },
    processing: false,
    async submit(config2 = {}) {
      const resp = await $precognition.client[resolveString(method)](
        resolveString(url),
        form.data(),
        resolveSubmitConfig(config2)
      );
      return resp;
    },
    validateFiles() {
      validator.validateFiles();
      return form;
    },
    validator() {
      return validator;
    }
  };
  form = reactive(form);
  return form;
}
