import { assertIsFetchResponse, assertIsServerGeneratedError, isFetchResponse, isSuccess, mergeConfig, resolveConfig, resolveStatusHandler, validatePrecognitionResponse } from "./utils/core.mjs";
import { defineNuxtPlugin, useRuntimeConfig } from "#imports";
export default defineNuxtPlugin((_nuxtApp) => {
  const precognitionConfig = useRuntimeConfig().public.nuxtLaravelPrecognition;
  let fetchClient = null;
  function getClient() {
    return fetchClient ?? $fetch.create({});
  }
  let successResolver = (response) => {
    return response.status === 204 && response.headers.get("precognition-success") === "true";
  };
  async function request(url, userConfig) {
    const config = resolveConfig(userConfig);
    if ((config.onBefore ?? (() => true))() === false)
      return Promise.resolve(null);
    if (config.onStart)
      config.onStart();
    try {
      const response = await getClient().raw(url, config);
      if (config.precognitive)
        validatePrecognitionResponse(response);
      const { status } = response;
      let payload = response;
      assertIsFetchResponse(payload, "Expected to receive FetchResponse");
      if (config.precognitive && config.onPrecognitionSuccess && successResolver(payload))
        payload = await Promise.resolve(config.onPrecognitionSuccess(payload) ?? payload);
      if (config.onSuccess && isSuccess(status)) {
        assertIsFetchResponse(payload, "Expected to receive FetchResponse for onSuccess handler");
        payload = await Promise.resolve(config.onSuccess(payload) ?? payload);
      }
      const statusHandler = resolveStatusHandler(config, status) ?? ((response2) => response2);
      payload = isFetchResponse(payload) ? statusHandler(payload) : payload;
      return isFetchResponse(payload) ? payload._data : payload;
    } catch (error) {
      assertIsServerGeneratedError(error);
      if (config.precognitive)
        validatePrecognitionResponse(error.response);
      const statusHandler = resolveStatusHandler(config, error.response.status) ?? ((_, error2) => Promise.reject(error2));
      return statusHandler(error.response, error);
    } finally {
      if (config.onFinish)
        config.onFinish();
    }
  }
  const client = {
    get: (url, data = {}, config = {}) => request(url, mergeConfig("get", data, config)),
    post: (url, data = {}, config = {}) => request(url, mergeConfig("post", data, config)),
    patch: (url, data = {}, config = {}) => request(url, mergeConfig("patch", data, config)),
    put: (url, data = {}, config = {}) => request(url, mergeConfig("put", data, config)),
    delete: (url, data = {}, config = {}) => request(url, mergeConfig("delete", data, config)),
    use(custom) {
      fetchClient = custom;
      return client;
    },
    client: () => getClient(),
    determineSuccessUsing(callback) {
      successResolver = callback;
      return client;
    }
  };
  return {
    provide: {
      precognition: {
        client,
        config: precognitionConfig
      }
    }
  };
});
