import { FetchError } from "ofetch";
import { createError } from "#imports";
export function validatePrecognitionResponse(response) {
  if (response.headers.get("precognition") === "true")
    return;
  throw createError({
    message: "Did not receive a Precognition response. Ensure you have the Precognition middleware in place for the route.",
    statusCode: 400
  });
}
export function resolveStatusHandler(config, code) {
  return {
    401: config.onUnauthorized,
    403: config.onForbidden,
    404: config.onNotFound,
    409: config.onConflict,
    422: config.onValidationError,
    423: config.onLocked
  }[code];
}
export function isSuccess(status) {
  return status >= 200 && status <= 300;
}
export function assertIsServerGeneratedError(error) {
  if (error instanceof FetchError && "response" in error && typeof error.response?.status === "number")
    return;
  if (error instanceof Error)
    throw error;
  throw createError("Expected server error.");
}
export function getHeaders(headers, name) {
  if (headers instanceof Headers)
    return headers.get(name);
  const newHeaders = new Headers(headers);
  return getHeaders(newHeaders, name);
}
export function setHeaders(headers, name, value) {
  if (headers instanceof Headers && getHeaders(headers, name)) {
    headers.set(name, value);
    return;
  }
  if (headers instanceof Headers) {
    headers.append(name, value);
    return;
  }
  if (!Array.isArray(headers)) {
    const key = Object.keys(headers).find((k) => k.toLowerCase() === name.toLowerCase()) ?? name;
    headers[key] = value;
    return;
  }
  const index = headers.findIndex(([headerName]) => headerName.toLowerCase() === name.toLowerCase());
  if (index >= 0) {
    headers[index] = [headers[index][0], value];
    return;
  }
  headers.push([name, value]);
}
export function resolveConfig(config) {
  const headers = config?.headers ?? {};
  if (config?.precognitive !== false)
    setHeaders(headers, "Precognition", "true");
  if (config?.validate)
    setHeaders(headers, "Precognition-Validate-Only", Array.from(config.validate).join());
  return {
    ...config,
    precognitive: config?.precognitive !== false,
    headers
  };
}
export function hasFiles(data) {
  if (typeof data !== "object")
    return false;
  if (!data)
    return false;
  if (isFile(data))
    return true;
  if (data instanceof FormData)
    return Array.from([...data.values()]).reduce((check, entry) => check || isFile(entry), false);
  return Object.values(data).some((value) => hasFiles(value));
}
export function isFile(value) {
  if (value instanceof Blob)
    return true;
  if (import.meta.server)
    return false;
  if (typeof File !== "undefined" && value instanceof File)
    return true;
  if (typeof FileList !== "undefined" && value instanceof FileList)
    return true;
  return false;
}
export function mergeConfig(method, data, config) {
  const baseConfig = {
    method,
    ...config,
    ...["get", "delete"].includes(method) ? {
      query: "query" in (config ?? {}) ? config?.query : (config?.queryAs ?? ((value) => value))(data ?? {})
    } : {
      body: "body" in (config ?? {}) ? config?.body : (config?.bodyAs ?? ((value) => value))(data ?? {})
    }
  };
  return baseConfig;
}
export function resolveString(value) {
  return typeof value === "function" ? value() : value;
}
export function resolveInitialData(data) {
  return typeof data === "function" ? data() : data;
}
export function assertIsFetchResponse(value, message = "Expected FetchResponse") {
  if (isFetchResponse(value))
    return;
  throw createError(message);
}
export function isFetchResponse(value) {
  return value instanceof Response;
}
