import { debounce, get, isEqual, merge, omit, set } from "lodash-es";
import { FetchError } from "ofetch";
import { isFile } from "../utils/core.mjs";
import { useNuxtApp } from "#imports";
export function createValidator(callback, initialData = {}) {
  const { $precognition } = useNuxtApp();
  const listeners = {
    errorsChanged: [],
    touchedChanged: [],
    validatingChanged: [],
    validatedChanged: []
  };
  let validateFiles = false;
  let validating = false;
  let errors = {};
  let validated = [];
  let debounceTimeoutDuration = 1500;
  let validator = createValidator2();
  const setValidating = (value) => {
    if (value !== validating) {
      validating = value;
      return listeners.validatingChanged;
    }
    return [];
  };
  const setValidated = (value) => {
    const uniqueNames = [...new Set(value)];
    if (validated.length !== uniqueNames.length || !uniqueNames.every((name) => validated.includes(name))) {
      validated = uniqueNames;
      return listeners.validatedChanged;
    }
    return [];
  };
  const valid = () => validated.filter((name) => typeof errors[name] === "undefined");
  let touched = [];
  const setTouched = (value) => {
    const uniqueNames = [...new Set(value)];
    if (touched.length !== uniqueNames.length || !uniqueNames.every((name) => touched.includes(name))) {
      touched = uniqueNames;
      return listeners.touchedChanged;
    }
    return [];
  };
  const setErrors = (value) => {
    const prepared = toValidationErrors(value);
    if (!isEqual(errors, prepared)) {
      errors = prepared;
      return listeners.errorsChanged;
    }
    return [];
  };
  const forgetError = (name) => {
    const newErrors = { ...errors };
    delete newErrors[resolveName(name)];
    return setErrors(newErrors);
  };
  const hasErrors = () => Object.keys(errors).length > 0;
  let oldData = initialData;
  let validatingData = null;
  let oldTouched = [];
  let validatingTouched = null;
  const parseData = (data) => validateFiles === false ? forgetFiles(data) : data;
  function createValidator2() {
    return debounce(() => {
      callback({
        get: (url, data = {}, config = {}) => $precognition.client.get(url, parseData(data), resolveConfig(config, data)),
        post: (url, data = {}, config = {}) => $precognition.client.post(url, parseData(data), resolveConfig(config, data)),
        patch: (url, data = {}, config = {}) => $precognition.client.patch(url, parseData(data), resolveConfig(config, data)),
        put: (url, data = {}, config = {}) => $precognition.client.put(url, parseData(data), resolveConfig(config, data)),
        delete: (url, data = {}, config = {}) => $precognition.client.delete(url, parseData(data), resolveConfig(config, data))
      }).catch((error) => error instanceof FetchError ? null : Promise.reject(error));
    }, debounceTimeoutDuration, { leading: true, trailing: true });
  }
  const setDebounceTimeout = (value) => {
    debounceTimeoutDuration = value;
    validator.cancel();
    validator = createValidator2();
  };
  function resolveConfig(config, data = {}) {
    const validate2 = Array.from(config.validate ?? touched);
    return {
      ...config,
      validate: validate2,
      timeout: config.timeout ?? 5e3,
      onValidationError: (response, fetchError) => {
        [
          ...setValidated([...validated, ...validate2])
        ].forEach((listener) => listener());
        if (typeof response._data === "object" && response.status === 422 && response._data && "errors" in response._data) {
          [
            ...setErrors(merge(omit({ ...errors }, validate2), response._data.errors))
          ].forEach((listener) => listener());
        }
        if (config.onValidationError)
          return config.onValidationError(response, fetchError);
        throw fetchError ?? new Error("Unknown error");
      },
      onSuccess: (response) => {
        setValidated([...validated, ...validate2]).forEach((listener) => listener());
        return Promise.resolve(response);
      },
      onPrecognitionSuccess: (response) => {
        [
          ...setValidated([...validated, ...validate2]),
          ...setErrors(omit({ ...errors }, validate2))
        ].forEach((listener) => listener());
        if (config.onPrecognitionSuccess)
          return config.onPrecognitionSuccess(response);
        return Promise.resolve(response);
      },
      onBefore: () => {
        const beforeValidationResult = (config.onBeforeValidation ?? ((previous, next) => {
          return !isEqual(previous, next);
        }))({ data, touched }, { data: oldData, touched: oldTouched });
        if (beforeValidationResult === false)
          return false;
        const beforeResult = (config.onBefore || (() => true))();
        if (beforeResult === false)
          return false;
        validatingTouched = touched;
        validatingData = data;
        return true;
      },
      onStart: () => {
        setValidating(true).forEach((listener) => listener());
        (config.onStart ?? (() => null))();
      },
      onFinish: () => {
        setValidating(false).forEach((listener) => listener());
        oldTouched = validatingTouched;
        oldData = validatingData;
        validatingTouched = validatingData = null;
        (config.onFinish ?? (() => null))();
      }
    };
  }
  const validate = (name, value) => {
    if (typeof name === "undefined") {
      validator();
      return;
    }
    if (isFile(value) && !validateFiles) {
      console.warn('Precognition file validation is not active. Call the "validateFiles" function on your form to enable it.');
      return;
    }
    name = resolveName(name);
    if (get(oldData, name) !== value)
      setTouched([name, ...touched]).forEach((listener) => listener());
    if (touched.length === 0)
      return;
    validator();
  };
  const form = {
    touched: () => touched,
    validate(input, value) {
      validate(input, value);
      return form;
    },
    touch(input) {
      const inputs = Array.isArray(input) ? input : [resolveName(input)];
      setTouched([...touched, ...inputs]).forEach((listener) => listener());
      return form;
    },
    validating: () => validating,
    valid,
    errors: () => errors,
    hasErrors,
    setErrors(value) {
      setErrors(value).forEach((listener) => listener());
      return form;
    },
    forgetError(name) {
      forgetError(name).forEach((listener) => listener());
      return form;
    },
    reset(...names) {
      if (names.length === 0) {
        setTouched([]).forEach((listener) => listener());
        setErrors({}).forEach((listener) => listener());
        setValidated([]).forEach((listener) => listener());
      } else {
        const newTouched = [...touched];
        names.forEach((name) => {
          if (newTouched.includes(name))
            newTouched.splice(newTouched.indexOf(name), 1);
          set(oldData, name, get(initialData, name));
        });
        setTouched(newTouched).forEach((listener) => listener());
      }
      return form;
    },
    setTimeout(value) {
      setDebounceTimeout(value);
      return form;
    },
    on(event, callback2) {
      listeners[event].push(callback2);
      return form;
    },
    validateFiles() {
      validateFiles = true;
      return form;
    }
  };
  return form;
}
export function toSimpleValidationErrors(errors) {
  return Object.keys(errors).reduce((carry, key) => ({
    ...carry,
    [key]: Array.isArray(errors[key]) ? errors[key][0] : errors[key]
  }), {});
}
export function toValidationErrors(errors) {
  return Object.keys(errors).reduce((carry, key) => ({
    ...carry,
    [key]: typeof errors[key] === "string" ? [errors[key]] : errors[key]
  }), {});
}
export function resolveName(name) {
  return typeof name !== "string" ? name.target.name : name;
}
function forgetFiles(data) {
  const newData = { ...data };
  Object.keys(newData).forEach((k) => {
    const name = k;
    const value = newData[name];
    if (value === null)
      return;
    if (isFile(value)) {
      delete newData[name];
      return;
    }
    if (Array.isArray(value)) {
      newData[name] = value.filter((value2) => !isFile(value2));
      return;
    }
    if (typeof value === "object") {
      newData[name] = forgetFiles(newData[name]);
    }
  });
  return newData;
}
